<?php
namespace Plaintracker1;

class Lib_Html
{
	public static function render( $m )
	{
		if( ! isset($m['@render']) ) return;

		call_user_func( $m['@render'], $m );

	// input? error?
		if( isset($m['error']) && strlen($m['error']) ){
			echo '<div><strong>' . esc_html( $m['error'] ) . '</strong></div>';
		}
	}

	public static function id( $m )
	{
		$ret = isset( $m['id'] ) ? $m['id'] : NULL;
		return $ret;
	}

	public static function input( $name, $value )
	{
		static $inputId = 1;

		$ret = [
			'value'		=> $value,
			'name'		=> $name,
			'id'			=> 'pw-input-' . $inputId++,
			'error'		=> '',
			'validate'	=> [],
			'@grab'		=> __CLASS__ . '::grab',
		];

		return $ret;
	}

	public static function grab( array $m, array $post )
	{
		$name = $m['name'];
		$ret = isset( $post[$name] ) ? $post[$name] : NULL;
		return $ret;
	}

	public static function grabTextarea( array $m, array $post )
	{
		$name = $m['name'];
		$ret = isset( $_POST[$name] ) ? sanitize_textarea_field( $_POST[$name] ) : NULL;
		return $ret;
	}

	public static function inputText( $name, $value )
	{
		$ret = [
			'@render'	=> __CLASS__ . '::renderInputText',
		] + static::input( $name, $value );

		return $ret;
	}

	public static function renderInputText( array $m )
	{
?>
<input type="text" name="<?php echo esc_attr($m['name']); ?>" value="<?php echo esc_attr($m['value']); ?>" id="<?php echo esc_attr($m['id']); ?>">
<?php
	}

	public static function inputTextarea( $name, $value )
	{
		$ret = [
			'@render'	=> __CLASS__ . '::renderInputTextarea',
			'@grab'		=> __CLASS__ . '::grabTextarea',
		] + static::input( $name, $value );

		return $ret;
	}

	public static function renderInputTextarea( array $m )
	{
?>
<textarea name="<?php echo esc_attr($m['name']); ?>" id="<?php echo esc_attr($m['id']); ?>"><?php echo esc_textarea($m['value']); ?></textarea>
<?php
	}

	public static function inputCheckbox( $name, $value )
	{
		$ret = [
			'@render'	=> __CLASS__ . '::renderInputCheckbox',
		] + static::input( $name, $value );

		return $ret;
	}

	public static function renderInputCheckbox( array $m )
	{
?>
<input type="checkbox" name="<?php echo esc_attr($m['name']); ?>" value="<?php echo esc_attr($m['value']); ?>" id="<?php echo esc_attr($m['id']); ?>">
<?php
	}

	public static function inputSelect( $name, $value, array $option  )
	{
		$ret = [
			'option'		=> $option,
			'@render'	=> __CLASS__ . '::renderInputSelect',
		] + static::input( $name, $value );

		return $ret;
	}

	public static function renderInputSelect( array $m )
	{
?>
<select name="<?php echo esc_attr($m['name']); ?>" id="<?php echo esc_attr($m['id']); ?>">
<?php foreach( $m['option'] as $k => $v ) : ?>
	<option value="<?php echo esc_attr($k); ?>"<?php if( $k == $m['value'] ) : ?> selected<?php endif; ?>><?php echo esc_html( $v ); ?></option>
<?php endforeach; ?>
</select>
<?php
	}

	public static function inputRadioSet( $name, $value, array $option  )
	{
		$ret = [
			'option'		=> $option,
			'@render'	=> __CLASS__ . '::renderInputRadioSet',
		] + static::input( $name, $value );

		return $ret;
	}

	public static function renderInputRadioSet( array $m )
	{
?>
<div class="pw-inline-list">
<?php foreach( $m['option'] as $k => $v ) : ?>
	<div>
		<label>
			<input type="radio" name="<?php echo esc_attr($m['name']); ?>" value="<?php echo esc_attr($k); ?>"<?php if( $k == $m['value'] ) : ?> checked<?php endif; ?>> <span><?php echo esc_html($v); ?></span>
		</label>
	</div>
<?php endforeach; ?>
</div>
<?php
	}

	public static function inputDate( $name, $value )
	{
		$ret = [
			'@render'	=> __CLASS__ . '::renderInputDate',
			'@grab'		=> __CLASS__ . '::grabInputDate',
		] + static::input( $name, $value );

		$option = range( 1, 12 );
		foreach( $option as $e ){
			$ret['text-month-' . $e] = Lib_TimeFormat::formatMonthName( $e );
		}

		return $ret;
	}

	public static function renderInputDate( array $m )
	{
		$t = Lib_Time::instance();

		$name = $m['name'];
		$value = $m['value'];

		$nameY = $name . '_y';
		$nameM = $name . '_m';
		$nameD = $name . '_d';

		if( NULL === $value ){
			$value = $t->setNow()->getDateDb();
		}
		$currentYear = $t->setDateDb( $value )->getYear();

		$valueY = substr( $value, 0, 4 );
		$valueM = substr( $value, 4, 2 );
		$valueD = substr( $value, 6, 2 );
?>

<fieldset>

<span class="pw-inline-list">

<span>
<?php $option = range( 1, 31 ); ?>
	<select name="<?php echo esc_attr($nameD); ?>"> 
	<?php foreach( $option as $e ) : ?>
		<option value="<?php echo esc_attr($e); ?>"<?php if( $e == $valueD ):?> selected<?php endif; ?>><?php echo esc_html($e); ?></option>
	<?php endforeach; ?>
	</select>
</span>

<span>
<?php $option = range( 1, 12 ); ?>
	<select name="<?php echo esc_attr($nameM); ?>"> 
	<?php foreach( $option as $e ) : ?>
		<option value="<?php echo esc_attr($e); ?>"<?php if( $e == $valueM ):?> selected<?php endif; ?>><?php echo esc_html($m['text-month-' . $e]); ?></option>
	<?php endforeach; ?>
	</select>
</span>

<span>
<?php $option = range( $currentYear - 10, $currentYear + 10 ); ?>
	<select name="<?php echo esc_attr($nameY); ?>"> 
	<?php foreach( $option as $e ) : ?>
		<option value="<?php echo esc_attr($e); ?>"<?php if( $e == $valueY ):?> selected<?php endif; ?>><?php echo esc_html($e); ?></option>
	<?php endforeach; ?>
	</select>
</span>
</span>

</fieldset>

<?php
	}

	public static function grabInputDate( array $m, array $post )
	{
		$name = $m['name'];

		$nameY = $name . '_y';
		$nameM = $name . '_m';
		$nameD = $name . '_d';

		$ret = NULL;
		if( isset($post[$nameY]) ){
			$y = $post[ $nameY ];
			$m = $post[ $nameM ];
			$d = $post[ $nameD ];
			$ret = $y . sprintf('%02d', $m) . sprintf('%02d', $d);
		}

		return $ret;
	}

// ----------------------------------------------------
// ValidatorRequired
// ----------------------------------------------------
	public static function validatorRequired( $msg = '__Required Field__' )
	{
		$ret = [
			'message' => $msg,
			'@validate'	=> __CLASS__ . '::validateRequired',
		];
		return $ret;
	}

	public static function validateRequired( array $m, $value )
	{
		if( is_array($value) ){
			return $value ? TRUE : FALSE;
		}
		return strlen($value) ? TRUE : FALSE;
	}

// ----------------------------------------------------
// ValidatorUniqueProp
// ----------------------------------------------------
	public static function validatorUniqueProp( array $ret = [] )
	{
		$ret += [
			'message'	=> '__This Value Already Exists__',
			'class'		=> '',
			'prop'		=> '',
			'skip'		=> NULL,
			'@validate'	=> __CLASS__ . '::validateUniqueProp',
		];
		return $ret;
	}

	public static function validateUniqueProp( array $m, $value )
	{
		$ret = TRUE;

		if( ! $m['class'] ) return $ret;
		if( ! $m['prop'] ) return $ret;

		$model = $m['class'];
		$propName = $m['prop'];
		$skip = $m['skip'];

		$q = $model::q();
		$q->where( $propName, '=', $value );
		$q->where( 'id', '<>', $skip );
		$q->limit( 1 );

		$matches = $model::find( $q );
		if( $matches ){
			$ret = FALSE;
		}

		return $ret;
	}

	public static function downloadFile( $file, $shortFile = NULL )
	{
		if( ob_get_contents() ){
			ob_end_clean();
		}

		if( NULL === $shortFile ){
			$shortFile = basename( $file );
		}

		$fileSize = filesize( $file );

		header("Type: application/force-download");
		header("Content-Type: application/force-download");
		header("Content-Length: $fileSize");
		header("Content-Transfer-Encoding: binary");
		header("Content-Disposition: attachment; filename=\"$shortFile\"");
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Connection: close");
		readfile( $file );
		exit;
	}

	public static function downloadData( $filename, $data )
	{
	// Try to determine if the filename includes a file extension.
	// We need it in order to set the MIME type
		if (FALSE === strpos($filename, '.')){
			return FALSE;
		}

	// Grab the file extension
		$x = explode('.', $filename);
		$extension = end($x);

		// Load the mime types
		$mimes = array();

		// Set a default mime if we can't find it
		if ( ! isset($mimes[$extension])){
			$mime = 'application/octet-stream';
		}
		else {
			$mime = (is_array($mimes[$extension])) ? $mimes[$extension][0] : $mimes[$extension];
		}

	// Generate the server headers
		if (strpos($_SERVER['HTTP_USER_AGENT'], "MSIE") !== FALSE){
			header('Content-Type: "'.$mime.'"');
			header('Content-Disposition: attachment; filename="'.$filename.'"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
			header("Content-Transfer-Encoding: binary");
			header('Pragma: public');
			header("Content-Length: ".strlen($data));
		}
		else {
			header('Content-Type: "'.$mime.'"');
			header('Content-Disposition: attachment; filename="'.$filename.'"');
			header("Content-Transfer-Encoding: binary");
			header('Expires: 0');
			header('Pragma: no-cache');
			header("Content-Length: ".strlen($data));
		}

		exit( $data );
	}
}