<?php
namespace Plaintracker1;

abstract class Lib_ModelSql
{
	const _CLASS = 'model';
	public $errors_ = [];
	public static $intTypes = [
		'INTEGER',
		'INT',
		// 'BIGINT',
		'TINYINT'
	];

	protected $_cached = [];

	final public static function tableName()
	{
		global $wpdb;

		if( is_multisite() ){
			// $shareDatabase = get_site_option( 'locatoraid_share_database', 0 );
			$shareDatabase = FALSE;
			$prefix = $shareDatabase ? $wpdb->base_prefix : $wpdb->prefix;
		}
		else {
			$prefix = $wpdb->prefix;
		}

		$prefix .= 'plaintracker_';

		$ret = static::table();
		$ret = str_replace( '__', $prefix, $ret );

		return $ret;
	}

/* overwrite in children */
	public static function table()
	{
		$ret = '';
		return $ret;
	}

	public static function fields()
	{
		$ret = [];
		return $ret;
	}

	public static function q()  
	{
		$q = Lib_Q::construct();
		return $q;
	}

/* migration */
	public static function version()
	{
		return 1;
	}

	public static function up1()
	{
		static::dbCreateTable();
	}

	public static function down1()
	{
		static::dbDropTable();
	}
/* end of migration */

/* database forge */
	public static function dbDropColumn( $name )
	{
		$tableName = static::tableName();
		$sql = 'ALTER TABLE ' . $tableName . ' DROP COLUMN ' . $name;
		return static::query( $sql, [] );
	}

	public static function dbDropTable()
	{
		$tableName = static::tableName();
		$sql = 'DROP TABLE ' . $tableName;
		return static::query( $sql, [] );
	}

	public static function dbAddColumn( $k, array $f )
	{
		$intTypes = static::$intTypes;
		$fields = static::fields();

		$sqlArg = [];

		$tableName = static::tableName();

		$sql = '';
		$sql .= 'ALTER TABLE ' . $tableName . ' ADD ';

		$sql .= $k . ' ' . $f['type'];
		$sql .= ( isset($f['null']) && $f['null'] ) ? ' NULL' : ' NOT NULL';

		if( isset($f['default']) ){
			$sql .= ' DEFAULT ';

			$isInt = ( isset($fields[$k]['type']) && in_array($fields[$k]['type'], $intTypes) ) ? TRUE : FALSE;
			if( $isInt ){
				$sql .= '%d';
			}
			else {
				$sql .= '%s';
			}
			$sqlArg[] = $f['default'];
		}

		return static::query( $sql, $sqlArg );
	}

	public static function dbCreateTable()
	{
		$intTypes = static::$intTypes;
		$fields = static::fields();

		$sqlArg = [];

		$tableName = static::tableName();

		$sql = '';
		$sql .= 'CREATE TABLE IF NOT EXISTS ' . $tableName . ' (';

		$s = [];
		$ks = [];
		reset( $fields );
		foreach( $fields as $k => $f ){
			$thisS = $k . ' ' . $f['type'];
			$thisS .= ( isset($f['null']) && $f['null'] ) ? ' NULL' : ' NOT NULL';
			if( isset($f['auto_increment']) && $f['auto_increment'] ){
				$thisS .= ' AUTO_INCREMENT';
			}
			if( isset($f['default']) ){
				$thisS .= ' DEFAULT ';

				$isInt = ( isset($fields[$k]['type']) && in_array($fields[$k]['type'], $intTypes) ) ? TRUE : FALSE;
				if( $isInt ){
					$thisS .= '%d';
				}
				else {
					$thisS .= '%s';
				}
				$sqlArg[] = $f['default'];
			}

			$s[] = $thisS;

			if( isset($f['key']) && $f['key'] ){
				$ks[ $k ] = $k;
			}
		}

		if( $ks ){
			$s[] = ' CONSTRAINT ' . join('_', $ks) . ' PRIMARY KEY(' . join(',', $ks) . ')';
		}

		$sql .= join( ', ', $s );
		$sql .= ')';

		return static::query( $sql, $sqlArg );
	}
/* end of database forge */

	public static function toArray( $m )
	{
		if( is_array($m) ) return $m;

		$ret = array();

		$vars = get_object_vars( $m );
		$fields = array_keys( $vars );

		foreach( $fields as $k ){
		// skip if ends with '_'
			if( '_' === substr($k, -1) ) continue;
			$v = $m->{$k};
			if( (NULL === $v) && ('id' === $k) ) continue;
			$ret[ $k ] = $v;
		}

		return $ret;
	}

	public static function fromArray( $m, array $array )
	{
		foreach( $array as $k => $v ){
			if( property_exists($m, $k) ){
				$m->{$k} = $v;
			}
		}
		return $m;
	}

	public static function whereId( Lib_Q $q, $m )
	{
		$id = is_array($m) ? $m['id'] : $m->id;
		$q->where( 'id', '=', $id );
		return $q;
	}

	public static function findAll()
	{
		$q = static::q();
		return static::find( $q );
	}

	public static function find( Lib_Q $q )
	{
		$ret = [];

		$res = static::load( $q );
		if( ! $res ) return $ret;

	// build models
		$bluePrintModel = static::construct();
		foreach( $res as $e ){
			if( ! Lib_Q::check($e, $q) ){
				continue;
			}

			$m = clone $bluePrintModel;
			$m = static::fromArray( $m, $e );
			if( ! $m ) continue;

			if( isset($m->id) ){
				$ret[ $m->id ] = $m;
			}
			elseif( method_exists($m, 'id') ){
				$ret[ $m->id() ] = $m;
			}
			else {
				$ret[] = $m;
			}
		}

		return $ret;
	}

	public static function count( $q )
	{
		$ret = 0;

		$sql = '';
		$sqlArg = [];

		$tableName = static::tableName();

	// select
		$sql = 'SELECT COUNT(*) AS count FROM ' . $tableName;

	// where
		list( $whereSql, $whereSqlArg ) = static::whereToSql( $q->getWhere() );
		$sql .= $whereSql;
		$sqlArg = array_merge( $sqlArg, $whereSqlArg );

		$ret = static::query( $sql, $sqlArg );
		if( $ret ){
			$ret = current( current($ret) );
			$ret = (int) $ret;
		}
		else {
			$ret = 0;
		}

		return $ret;
	}

	public static function findById( $id )
	{
		// if( ! is_array($id) ){
			// if( isset($this->_cached[$id]) ){
				// return $this->_cached[$id];
			// }
		// }

		$q = static::q();
		$q->where( 'id', '=', $id );

		if( ! is_array($id) ){
			$q->limit( 1 );
		}

		$ret = static::find( $q );

		if( ! is_array($id) ){
			$ret = array_shift( $ret );
			// $this->_cached[$id] = $ret;
		}

		return $ret;
	}

	public function findCache( Lib_Q $q, $parentFunc )
	{
		if( NULL === $this->_all ){
		// load all
			$q2 = $this->self->q();
			$this->_all = call_user_func( $parentFunc, $q2 );
		}

		$ret = $this->_all;
		$ret = $this->q->filter( $ret, $q );

		return $ret;
	}

// commands
	public function validateCreate( $model )
	{
	// set $model->errors_ if needed
		return $model;
	}

	public static function create( $m )
	{
		$values = static::toArray( $m );

		$sql = '';
		$sqlArg = [];

		$tableName = static::tableName();

	// insert
		$sql = 'INSERT INTO ' . $tableName;

	// fields
		$sf = [];
		$sv = [];
		foreach( $values as $k => $v ){
			$sf[] = $k;
			$sqlArg[] = $v;

			$isInt = ( isset($fields[$k]['type']) && in_array($fields[$k]['type'], $intTypes) ) ? TRUE : FALSE;
			if( $isInt ){
				$sv[] = '%d';
			}
			else {
				$sv[] = '%s';
			}
		}
		$sql .= ' (' . join(', ', $sf) . ') VALUES (' . join(', ', $sv) . ')';

		$insertId = static::query( $sql, $sqlArg );

		if( is_array($m) ){
			$m['id'] = $insertId;
		}
		elseif( property_exists($m, 'id') ){
			$m->id = $insertId;
		}

		return $m;
	}

	public static function createMany( array $models )
	{
		$arrayOfValues = array();
		foreach( $models as $model ){
			$values = static::toArray( $model );
			if( ! $values ) continue;
			$arrayOfValues[] = $values;
		}

		$intTypes = static::$intTypes;
		$fields = static::fields();

		$tableName = static::tableName();

		$sqls = [];
		$sqlArgs = [];

	// split into batches
		$batchSize = 100;
		// $batchSize = 2;
		$batchCount = ceil( count($arrayOfValues) / $batchSize );

		for( $ii = 0; $ii < $batchCount; $ii++ ){
			$thisArrayOfValues = array_slice( $arrayOfValues, $batchSize * $ii, $batchSize );

			$sql = '';
			$sqlArg = [];

		// insert
			$sql = 'INSERT INTO ' . $tableName;

			$sf = [];
			$sv = [];

		// fields
			reset( $thisArrayOfValues );
			foreach( $thisArrayOfValues as $values ){
				foreach( $values as $k => $v ){
					if( isset($sf[$k]) ) continue;
					$sf[$k] = $k;
				}
			}

			reset( $thisArrayOfValues );
			foreach( $thisArrayOfValues as $values ){
				reset( $sf );
				$thisSv = [];
				foreach( $sf as $k ){
					if( ! isset($values[$k]) ){
						$thisSv[] = 'NULL';
					}
					else {
						$sqlArg[] = $values[$k];
						$isInt = ( isset($fieldsConf[$k]['type']) && in_array($fieldsConf[$k]['type'], $intTypes) ) ? TRUE : FALSE;
						if( $isInt ){
							$thisSv[] = '%d';
						}
						else {
							$thisSv[] = '%s';
						}
					}
				}
				$sv[] = '(' . join(', ', $thisSv) . ')';
			}

			$sql .= ' (' . join(', ', $sf) . ') VALUES ' . join(', ', $sv);

			$sqls[] = $sql;
			$sqlArgs[] = $sqlArg;
		}

// _print_r( $sqls );
// _print_r( $sqlArgs );
// exit;

		$ret = 0;
		for( $ii = 0; $ii < count($sqls); $ii++ ){
			$sql = $sqls[$ii];
			$sqlArg = $sqlArgs[$ii];
			$thisRet = static::query( $sql, $sqlArg );
			$ret += $thisRet;
		}

		return $ret;
	}

	public static function dumbCreateMany( array $arrayOfValues )
	{
		$ret = 0;

		foreach( $arrayOfValues as $values ){
			static::create( $values );
			$ret++;
		}

		return $ret;
	}

	public static function update( $oldM, $newM )
	{
		$newValues = static::toArray( $newM );
		$oldValues = static::toArray( $oldM );
// _print_r( $values );
// _print_r( $oldValues );
// exit;
	// update diff
		// $diff = array_diff_assoc( $values, $oldValues );
	// need to handle arrays too

		$values = [];
		foreach( $newValues as $k => $v ){
			if( array_key_exists($k, $oldValues) && ($oldValues[$k] == $v) ){
				continue;
			}
			$values[ $k ] = $v;
		}

		if( ! $values ){
			return $newM;
		}

// _print_r( $diff );
// exit;

		$sql = '';
		$sqlArg = [];

		$tableName = static::tableName();

		$q = static::q();
		$q = static::whereId( $q, $oldM );

		$intTypes = static::$intTypes;
		$fields = static::fields();

	// update
		$sql = 'UPDATE ' . $tableName;

	// fields
		$s = [];
		foreach( $values as $k => $v ){
			$thisS = $k . '=';
			$isInt = ( isset($fieldsConf[$k]['type']) && in_array($fieldsConf[$k]['type'], $intTypes) ) ? TRUE : FALSE;
			if( $isInt ){
				$thisS .= '%d';
			}
			else {
				$thisS .= '%s';
			}
			$s[] = $thisS;
			$sqlArg[] = $v;
		}
		$sql .= ' SET ' . join(', ', $s);

	// where
		list( $whereSql, $whereSqlArg ) = static::whereToSql( $q->getWhere() );
		$sql .= $whereSql;
		$sqlArg = array_merge( $sqlArg, $whereSqlArg );

		static::query( $sql, $sqlArg );

		return $newM;
	}

	public static function load( Lib_Q $q )
	{
		$ret = [];

		$sql = '';
		$sqlArg = [];

		$tableName = static::tableName();

	// select
		$sql = 'SELECT * FROM ' . $tableName;

	// where
		list( $whereSql, $whereSqlArg ) = static::whereToSql( $q->getWhere() );
		$sql .= $whereSql;
		$sqlArg = array_merge( $sqlArg, $whereSqlArg );

	// orderby
		$orderBy = $q->getOrderBy();
		if( $orderBy ){
			$s = [];
			foreach( $orderBy as $w ){
				list( $name, $direction ) = $w;
				$s[] = $name . ' ' . $direction;
			}
			$sql .= ' ORDER BY ' . join( ', ', $s );
		}

	// limit
		$limit = $q->getLimit();
		$offset = $q->getOffset();
		if( $offset && $limit ){
			$sql .= ' LIMIT ' . $offset . ', ' . $limit;
		}
		elseif( $limit ){
			$sql .= ' LIMIT ' . $limit;
		}

		$res = static::query( $sql, $sqlArg );

		if( $res ){
			foreach( $res as $e ){
				if( isset($e['id']) ){
					$ret[ $e['id'] ] = $e;
				}
				else {
					$ret[] = $e;
				}
			}
		}

		return $ret;
	}

	public static function delete( $m )
	{
		$q = static::q();
		$q = static::whereId( $q, $m );

		$where = $q->getWhere();
		if( ! $where ){
			exit( __METHOD__ . ': ' . __LINE__ . ': cannot proceed without conditions!' );
		}

		$sql = '';
		$sqlArg = [];

		$tableName = static::tableName();

	// select
		$sql = 'DELETE FROM ' . $tableName;

	// where
		list( $whereSql, $whereSqlArg ) = static::whereToSql( $q->getWhere() );
		$sql .= $whereSql;
		$sqlArg = array_merge( $sqlArg, $whereSqlArg );

		static::query( $sql, $sqlArg );

		return $m;
	}

	public static function deleteMany( $q )
	{
		$where = $q->getWhere();
		if( ! $where ){
			exit( __METHOD__ . ': ' . __LINE__ . ': cannot proceed without conditions!' );
		}

		$tableName = static::tableName();

		$sql = '';
		$sqlArg = [];

	// select
		$sql = 'DELETE FROM ' . $tableName;

	// where
		list( $whereSql, $whereSqlArg ) = static::whereToSql( $q->getWhere() );
		$sql .= $whereSql;
		$sqlArg = array_merge( $sqlArg, $whereSqlArg );

		$ret = static::query( $sql, $sqlArg );

		return $ret;
	}

	public static function whereToSql( array $where )
	{
		$intTypes = static::$intTypes;
		$fieldsConf = static::fields();

		$sql = '';

		$s = [];
		$sqlArgs = [];

		foreach( $where as $w ){
			list( $name, $compare, $value ) = $w;
			$isInt = ( isset($fieldsConf[$name]['type']) && in_array($fieldsConf[$name]['type'], $intTypes) ) ? TRUE : FALSE;
			if( 'LIKE' === $compare ) $compare = ' LIKE ';

			if( $isInt ){
				$thisV = '%d';
			}
			else {
				$thisV = '%s';
			}

			if( is_array($value) && ('=' == $compare) ){
				$compare = ' IN ';
				$thisV = array_fill( 0, count($value), $thisV );
				$thisV = '(' . join( ',', $thisV ) . ')';
				foreach( $value as $v ) $sqlArgs[] = $v;
			}
			else {
				$sqlArgs[] = $value;
			}

			$thisS = $name . $compare . $thisV;
			$s[] = $thisS;
		}

		if( ! $s ){
			$s[] = '1=%d';
			$sqlArgs[] = 1;
		}

		$sql .= ' WHERE ' . join( ' AND ', $s );
		$ret = [ $sql, $sqlArgs ];

		return $ret;
	}

/* query wpdb */
	public static function query( $sql, array $arg )
	{
		global $wpdb;

		$isSelect = FALSE;
		$isInsert = FALSE;
		if( preg_match( '/^\s*insert\s/i', $sql ) ){
			$isInsert = TRUE;
		}
		elseif( preg_match( '/^\s*(select|show)\s/i', $sql ) ){
			$isSelect = TRUE;
		}

		if( $arg ){
			$sql = $wpdb->prepare( $sql, $arg );
		}

		if( $isSelect ){
			$ret = $wpdb->get_results( $sql, ARRAY_A );
		}
		else {
			$ret = $wpdb->query( $sql );
			if( $isInsert && (1 == $ret) ){
				$ret = $wpdb->insert_id;
			}
		}

		if( $wpdb->last_error ){
			$ret = FALSE;
			return $ret;
			exit( $wpdb->last_error . '<br/>' . $sql );
		}

		return $ret;
	}
}