<?php
namespace Plaintracker1;

class Lib_Time extends \DateTime
{
	public static $instance;

	public static $weekStartsOn = 0;
	public static $timezone = '';

	public static function instance()
	{
		if( NULL === self::$instance ){
			self::$instance = new static;
		}
		return self::$instance;
	}

	public function getDayOfWeekOccurenceInMonth( $date )
	{
		$this->setDateDb( $date );
		$month = $this->getMonth();
		$rexMonth = $month;

		$ret = 0;
		while( $rexMonth == $month ){
			$ret++;
			$this->modify( '-1 week' );
			$rexMonth = $this->getMonth();
		}

		return $ret;
	}

	public function getDayOfWeekOccurenceInMonthFromEnd( $date )
	{
		$this->setDateDb( $date );
		$month = $this->getMonth();
		$rexMonth = $month;

		$ret = 0;
		while( $rexMonth == $month ){
			$ret--;
			$this->modify( '+1 week' );
			$rexMonth = $this->getMonth();
		}

		return $ret;
	}

	public function addSeconds( $dateTime, $duration )
	{
		static $cache = array();

		$key = $dateTime;
		$key .= ( $duration > 0 ) ? '+' . $duration : '-' . (-$duration);

		if( isset($cache[$key]) ){
			return $cache[$key];
		}

		$this->setDateTimeDb( $dateTime );

		if( $duration > 0 ){
			$this->modify( '+ ' . $duration . ' seconds' );
		}
		else {
			$this->modify( '- ' . (- $duration) . ' seconds' );
		}

		$thisDateTime = $this->getDateTimeDb();
		$cache[$key] = $thisDateTime;

		return $cache[$key];
	}

	public function dateTime( $dateDb, $timeSeconds )
	{
		static $cache = array();
		// static $secToDb = NULL;
		static $secToDb = array();

		$key = $dateDb . '-' . $timeSeconds;
		if( isset($cache[$key]) ){
			return $cache[$key];
		}

		if( NULL === $secToDb ){
			$minutes = array( '00', '05', '10', '15', '20', '25', '30', '35', '40', '45', '50', '55' );
			$hours = array( '00','01','02','03','04','05','06','07','08','09','10','11','12','13','14','15','16','17','18','19','20','21','22','23' );

			$secToDb = array();
			for( $h = 0; $h < count($hours); $h++ ){
				for( $m = 0; $m < count($minutes); $m++ ){
					$sec = $h * 60 * 60 + $m * 60;
					$db = $hours[$h] . $minutes[$m];
					$secToDb[ $sec ] = $db;
				}
			}
			// $secToDb = array();
		}

		if( isset($secToDb[$timeSeconds]) ){
			$ret = $dateDb . $secToDb[$timeSeconds];
		}
		else {
			$ret = $this->setDateDb( $dateDb )
				->modify( '+ ' . $timeSeconds . ' seconds' )
				->getDateTimeDb()
				;
		}

		$cache[$key] = $ret;
		return $cache[$key];
	}

	public function splitToWeeks( array $dates )
	{
		$return = array();

		$thisWeek = array();
		foreach( $dates as $date ){
			$this->setDateDb( $date );
			$weekNo = $this->getWeekNo();
			if( ! isset($return[$weekNo]) ){
				$return[$weekNo] = array();
			}
			$return[$weekNo][] = $date;
		}

		return $return;
	}

	public function modify( $modify )
	{
		parent::modify( $modify );
		return $this;
	}

	public function smartModifyDown( $modify )
	{
		$this->modify( $modify );

		list( $qty, $measure ) = explode( ' ', $modify );
		switch( $measure ){
			case 'days':
				$this->setStartDay();
				break;
			case 'weeks':
				$this->setStartWeek();
				break;
			case 'months':
				$this->setStartMonth();
				break;
		}

		return $this;
	}

	public function smartModifyUp( $modify )
	{
		$this->modify( $modify );

		list( $qty, $measure ) = explode( ' ', $modify );
		switch( $measure ){
			case 'days':
				$this->setEndDay();
				break;
			case 'weeks':
				$this->setEndWeek();
				break;
			case 'months':
				$this->setEndMonth();
				break;
		}

		return $this;
	}

	/* date2 - date1 */
	public function getDifferenceInDays( $date1, $date2 )
	{
		$ts1 = $this->setDateDb( $date1 )->getTimestamp();
		$ts2 = $this->setDateDb( $date2 )->getTimestamp();

		$tsDiff = $ts2 - $ts1;
		$day = 24 * 60 * 60;

		$dayDiff = floor( $tsDiff / $day );
		return $dayDiff;
	}

	public function setTimezone( $tz )
	{
		if( is_array($tz) )
			$tz = $tz[0];

		if( ! $tz )
			$tz = date_default_timezone_get();

		$this->timezone = $tz;

		try {
			$tzObject = new DateTimeZone( $tz );
			parent::setTimezone( $tzObject );
		}
		catch( Exception $e ){
			exit( "WRONG TIMEZONE: '$tz'" );
		}
	}

	public function setTimestamp( $ts )
	{
		if( ! strlen($ts) ){
			$ts = 0;
		}

		if( function_exists('date_timestamp_set') ){
			parent::setTimestamp( $ts );
		}
		else {
			parent::__construct( '@' . $ts );
		}

		return $this;
	}

	public function setNow()
	{
		$this->setTimestamp( time() );
		return $this;
	}

	public function formatDateDb()
	{
		return $this->getDateDb();
	}

	public function getDateDb( $dateTimeDb = NULL )
	{
		if( NULL === $dateTimeDb ){
			$dateFormat = 'Ymd';
			$return = $this->format( $dateFormat );
		}
		else {
			$return = substr( $dateTimeDb, 0, 8 );
		}
		return $return;
	}

	public function setDateDb( $date )
	{
		list( $year, $month, $day ) = $this->_splitDate( $date );
		$year = (int) $year;
		$month = (int) $month;
		$day = (int) $day;

		$this->setDate( $year, $month, $day );
		$this->setTime( 0, 0, 0 );
		return $this;
	}

	public function setDateTimeDb( $datetime )
	{
		$date = substr($datetime, 0, 8);
		$this->setDateDb( $date );

		$hours = substr($datetime, 8, 2);
		$minutes = substr($datetime, 10, 2);
		$this->setTime( (int) $hours, (int) $minutes, 0 );

		return $this;
	}

	protected function _splitDate( $string )
	{
		$year = substr( $string, 0, 4 );
		$month = substr( $string, 4, 2 );
		$day = substr( $string, 6, 4 );
		$return = array( $year, $month, $day );
		return $return;
	}

	public function getDateTimeDb()
	{
		$date = $this->getDateDb();
		$time = $this->getTimeDb();
		$return = $date . $time;
		return $return;
	}

	public function getEndDateTimeDb( $date = NULL )
	{
		if( NULL === $date ){
			$date = $this->getDateDb();
		}
		$time = '2400';
		$return = $date . $time;
		return $return;
	}

	public function formatDateTimeDb2()
	{
		$return = $this->format('Y-m-d H:i:s');
		return $return;
	}

	public function getTimeDb( $dateTimeDb = NULL )
	{
		if( NULL === $dateTimeDb ){
			$h = $this->format('G');
			$m = $this->format('i');

			$h = str_pad( $h, 2, 0, STR_PAD_LEFT );
			$m = str_pad( $m, 2, 0, STR_PAD_LEFT );

			$ret = $h . $m;
		}
		else {
			$ret = substr( $dateTimeDb, 8, 4 );
		}

		return $ret;
	}

	public function setStartDay()
	{
		$this->setTime( 0, 0, 0 );
		return $this;
	}

	public function setEndDay()
	{
		// $this
			// ->setStartDay()
			// ->modify('+1 day')
			// ;
		$this
			->setTime( 23, 59, 59 )
			;
		return $this;
	}

	public function getTimeInDay( $dateTimeDb = NULL )
	{
		$ret = NULL;

		if( NULL === $dateTimeDb ){
			$timestamp = $this->getTimestamp();

			$this->setStartDay();
			$timestamp2 = $this->getTimestamp();

			$ret = $timestamp - $timestamp2;

			$this->setTimestamp( $timestamp );
		}
		else {
			
		}

		return $ret;
	}

	public function setStartWeek()
	{
		$this->setStartDay();
		$weekDay = $this->getWeekday();

		$wso = static::$weekStartsOn;
		while( $weekDay != $wso ){
			$this->modify( '-1 day' );
			$weekDay = $this->getWeekday();
		}

		return $this;
	}

	public function isStartWeek( $dateDb )
	{
		return ( $this->getWeekday($dateDb) == static::$weekStartsOn ) ? TRUE : FALSE;
	}

	public function setEndWeek()
	{
		$this->setStartDay();
		$this->modify( '+1 day' );
		$weekDay = $this->getWeekday();

		$wso = static::$weekStartsOn;
		while( $weekDay != $wso ){
			$this->modify( '+1 day' );
			$weekDay = $this->getWeekday();
		}

		$this
			->modify( '-1 day' )
			->setEndDay()
			;
		return $this;
	}

	public function setStartMonth()
	{
		$year = $this->format('Y');
		$month = $this->format('m');
		$day = '01';

		$date = $year . $month . $day;
		$this
			->setDateDb( $date )
			->setTime( 0, 0, 0 )
			;

		return $this;
	}

	public function setEndMonth()
	{
		$currentMonth = $this->format('m');
		$nextMonth = $currentMonth;

		while( $currentMonth == $nextMonth ){
			$this->modify('+28 days');
			$nextMonth = $this->format('m');
		}

		$year = $this->format('Y');
		$month = $this->format('m');
		$day = '01';

		$date = $year . $month . $day;
		$this
			->setDateDb( $date )
			->modify('-1 day')
			->setEndDay()
			;

		return $this;
	}

	public function setStartYear()
	{
		$year = $this->format('Y');
		$month = '01';
		$day = '01';

		$date = $year . $month . $day;
		$this
			->setDateDb( $date )
			->setTime( 0, 0, 0 )
			;

		return $this;
	}

	public function setEndYear()
	{
		$this
			->setStartYear()
			->modify('+1 year')
			->modify('-1 day')
			;

		return $this;
	}

	public function getYear( $dateTimeDb = NULL )
	{
		if( NULL !== $dateTimeDb ){
			$this->setDateTimeDb( $dateTimeDb );
		}
		$ret = $this->format('Y');
		return $ret;
	}

	public function getDay( $dateTimeDb = NULL )
	{
		if( NULL !== $dateTimeDb ){
			$this->setDateTimeDb( $dateTimeDb );
		}
		$ret = $this->format('j');
		return $ret;
	}

	public function getWeekday( $dateDb = NULL )
	{
		static $cache = array();

		if( NULL !== $dateDb ){
			if( isset($cache[$dateDb]) ){
				return $cache[$dateDb];
			}
			$this->setDateDb( $dateDb );
		}

		$ret = $this->format('w');

		if( NULL !== $dateDb ){
			$cache[ $dateDb ] = $ret;
		}

		return $ret;
	}

	public function getMonthMatrix( $skipWeekdays = array(), $overlap = FALSE )
	{
		// $overlap = TRUE; // if to show dates of prev/next month
		// $overlap = FALSE; // if to show dates of prev/next month

		$matrix = array();
		$currentMonthDay = 0;

		$currentDate = $this->formatDateDb();
		$thisMonth = substr( $currentDate, 4, 2 );

		$this->setStartMonth();
		if( $overlap ){
			$this->setStartWeek();
		}
		$startDate = $this->formatDateDb();

		$this
			->setDateDb( $currentDate )
			->setEndMonth()
			;
		if( $overlap ){
			$this->setEndWeek();
		}
		$this->modify('-1 second');

		$endDate = $this->formatDateDb();

		$rexDate = $startDate;
		if( $overlap ){
			$this->setDateDb( $startDate );
			$this->setStartWeek();
			$rexDate = $this->formatDateDb();
		}

		$this->setDateDb( $startDate );
		$this->setStartWeek();
		$rexDate = $this->formatDateDb();

		$this->setDateDb( $rexDate );
		while( $rexDate <= $endDate ){
			$week = array();
			$weekSet = FALSE;
			$thisWeekStart = $rexDate;

			for( $weekDay = 0; $weekDay <= 6; $weekDay++ ){
				$thisWeekday = $this->getWeekday();
				$setDate = $rexDate;

				if( ! $overlap ){
					if( 
						( $rexDate > $endDate ) OR
						( $rexDate < $startDate )
						){
						$setDate = NULL;
						}
				}

				// $week[ $thisWeekday ] = $setDate;

				if( (! $skipWeekdays) OR (! in_array($thisWeekday, $skipWeekdays)) ){
					if( NULL !== $setDate ){
						$rexMonth = substr( $setDate, 4, 2 );

						if( ! $overlap ){
							if( $rexMonth != $thisMonth ){
								$setDate = NULL;
							}
						}
					}

					$wki = $this->getWeekday();
					$week[ $wki ] = $setDate;
					if( NULL !== $setDate ){
						$weekSet = TRUE;
					}
				}

				$this->modify('+1 day');
				$rexDate = $this->formatDateDb();
			}

			if( $weekSet )
				$matrix[$thisWeekStart] = $week;
		}

		return $matrix;
	}

	public function getParts()
	{
		$full = $this->formatDateTimeDb();

		$year = substr( $full, 0, 4 );
		$month = substr( $full, 4, 2 );
		$day = substr( $full, 6, 2 );
		$hour = substr( $full, 8, 2 );
		$min = substr( $full, 10, 2 );

		$return = array( $year, $month, $day, $hour, $min );
		return $return;
	}

	public function getSortedWeekdays()
	{
		$return = array( 0, 1, 2, 3, 4, 5, 6 );
		$return = $this->sortWeekdays( $return );
		return $return;
	}

	public function sortWeekdays( $wds )
	{
		$return = array();
		$later = array();

		sort( $wds );
		reset( $wds );
		foreach( $wds as $wd ){
			if( $wd < static::$weekStartsOn )
				$later[] = $wd;
			else
				$return[] = $wd;
		}
		$return = array_merge( $return, $later );
		return $return;
	}

	public function getDuration( $dateTime1, $dateTime2 )
	{
		if( $dateTime1 == $dateTime2 ){
			return 0;
		}

		// $timestamp1 = $this->getTimestamp();
		$timestamp1 = $this->setDateTimeDb( $dateTime1 )->getTimestamp();
		$timestamp2 = $this->setDateTimeDb( $dateTime2 )->getTimestamp();

		$ret = abs( $timestamp2 - $timestamp1 );
		return $ret;
	}

	public function getMonth()
	{
		$month = $this->format('n');
		return $month;
	}

	public function getWeekNo( $date = NULL )
	{
		if( NULL !== $date ){
			$this->setDateDb( $date );
		}

		$ret = $this->format('W'); // but it works out of the box for week starts on monday
		$weekday = $this->getWeekday();
		if( ! $weekday ){ // sunday
			if( ! static::$weekStartsOn ){
				$ret = $ret + 1;
			}
		}

		return $ret;
	}

	public function getAllDates( $startDate, $endDate, $withStartEnd = FALSE )
	{
		static $cache = array();
		$key = $startDate . '-' . $endDate;
		$key .= $withStartEnd ? '-1' : '-0';
		if( isset($cache[$key]) ){
			return $cache[$key];
		}

		$return = array();

		$rexDate = $startDate;
		$this->setDateDb( $rexDate );
		while( $rexDate <= $endDate ){
			if( $withStartEnd ){
				$startDateTime = $this->dateTime( $rexDate, 0 );
				$endDateTime = $this->getEndDateTimeDb( $rexDate );
				$return[ $rexDate ] = array( $startDateTime, $endDateTime );
			}
			else {
				$return[] = $rexDate;
			}
			$rexDate = $this->getNextDate( $rexDate );
		}

		$cache[$key] = $return;
		return $cache[$key];
	}

	public function getNextDate( $date )
	{
		static $cache = array();
		$key = $date;
		if( isset($cache[$key]) ){
			return $cache[$key];
		}

		$ret = $this->setDateDb( $date )->modify( '+1 day' )->getDateDb();
		$cache[$key] = $ret;
		return $cache[$key];
	}

	public function getPrevDate( $date )
	{
		static $cache = array();
		$key = $date;
		if( isset($cache[$key]) ){
			return $cache[$key];
		}

		$ret = $this->setDateDb( $date )->modify( '-1 day' )->getDateDb();
		$cache[$key] = $ret;
		return $cache[$key];
	}
}