<?php
namespace Plaintracker1;

class Model_Install extends Lib_ModelSql
{
	static $loaded = NULL;

	public static function table()
	{
		$ret = '__install';
		return $ret;
	}

	public static function modules()
	{
		$ret = array();

		$ret[ 'install' ] = __CLASS__;
		$ret[ 'activity' ] = Model_Activity::class;
		$ret[ 'record' ] = Model_Record::class;
		$ret[ 'setting' ] = Model_Setting::class;

		return $ret;
	}

	public static function up()
	{
		$modules = static::modules();

		foreach( $modules as $id => $module ){
			$needVersion = $module::version();
			$installedVersion = static::get( $id );
			if( ! $installedVersion ) $installedVersion = 0;

			for( $ver = ($installedVersion + 1); $ver <= $needVersion; $ver++ ){
				call_user_func( [$module, 'up' . $ver] );
				static::set( $id, $ver );
			}
		}
	}

	public static function down()
	{
		$modules = static::modules();
		$modules = array_reverse( $modules );

		foreach( $modules as $id => $module ){
			$needVersion = 1;
			$installedVersion = static::get( $id );
			if( ! $installedVersion ) $installedVersion = 0;

			for( $ver = $installedVersion; $ver >= $needVersion; $ver-- ){
				call_user_func( [$module, 'down' . $ver] );
				static::set( $id, $ver );
			}
		}
	}

	public static function fields()
	{
		$ret = [
			'id'			=> [ 'type' => 'VARCHAR(64)', 'null' => FALSE, 'key' => TRUE ],
			'version'	=> [ 'type' => 'INTEGER', 'null'	=> FALSE ],
		];
		return $ret;
	}

	public static function get( $id )
	{
		if( NULL === static::$loaded ){
			$q = static::q();
			$data = static::load( $q );
			static::$loaded = [];
			foreach( $data as $e ){
				static::$loaded[ $e['id'] ] = $e['version'];
			}
		} 

		$values = static::$loaded;
		$ret = array_key_exists( $id, $values ) ? $values[ $id ] : 0;

		return $ret;
	}

	public static function set( $id, $version )
	{
		if( NULL === static::$loaded ){
			$q = static::q();
			static::$loaded = static::load( $q );
		} 

		$newM = [ 'id' => $id, 'version' => $version ];

		if( array_key_exists($id, static::$loaded) ){
			if( $version != static::$loaded[$id] ){
				$oldM = [ 'id' => $id, 'version' => static::$loaded[$id] ];
				static::update( $oldM, $newM );
				static::$loaded[$id] = $version;
			}
		}
		else {
			$newM = [ 'id' => $id, 'version' => $version ];
			static::create( $newM );
		}

		return $version;
	}
}