<?php
namespace Plaintracker1;

class Page_ToolImport
{
	public static function title( array $x )
	{
		return '__Import__';
	}

	public static function menu( array $x )
	{
		$ret = [];
		$ret['tool'] = '&larr; ' . '__Tools__';
		return $ret;
	}

	public static function post( array $x, array $values )
	{
		$keepCurrent = $values['keep_current'] ? TRUE : FALSE;

		$strings = $values['record'];
		$strings = explode( "\n", $strings );

		$activities = Model_Activity::findAll();
		$deleteActivity = [];
		$activityByName = [];
		foreach( $activities as $m ){
			$activityByName[ $m->title ] = $m->id;
			$deleteActivity[ $m->id ] = $m->id;
		}

		$models = [];

		$separator = $values['separator'];
		$fields = [ $values['field1'], $values['field2'], $values['field3'] ];

		$createMissingActivity = TRUE;

		foreach( $strings as $string ){
			$string = trim( $string );
			if( ! strlen($string) ) continue;

			switch( $separator ){
				case 's':
					$parts = preg_split( '/[\s+]/u', $string );
					break;

				default:
					$parts = explode( $separator, $string );
					break;
			}

			$props = [];
			foreach( $fields as $f ){
				$props[ $f ] = array_shift( $parts );
			}

			extract( $props );

			$activity = trim( $activity );

			if( ! isset($activityByName[$activity]) ){
				if( $createMissingActivity ){
					$newActivity = Model_Activity::construct();
					$newActivity->title = $activity;
					$newActivity = Model_Activity::create( $newActivity );
					$activityByName[$activity] = $newActivity->id;
				}
				else {
					$errs = [ 'record' => "'$activity' activity not registered" ]; 
					// $x->resp['error'] = $errs;
					return $x;
				}
			}

			$activityId = $activityByName[$activity];
			unset( $deleteActivity[$activityId] );

			$date = trim( $date );
			$date = preg_replace( '/[^0-9]/', '', $date );
 
			$qty = trim( $qty );

			$m = Model_Record::construct();
			$m->date = $date;
			$m->qty = $qty;
			$m->activity_id = $activityId;

			$models[] = $m;
		}

		if( ! $keepCurrent ){
			$q = Model_Record::q();
			$q->where( 1, '=', 1 );
			Model_Record::deleteMany( $q );

			if( $deleteActivity ){
				$q = Model_Activity::q();
				$q->where( 'id', '=', $deleteActivity );
				Model_Activity::deleteMany( $q );
			}
		}

		Model_Record::createMany( $models );

		$x['redirect'] = 'record';
		return $x;
	}

	public static function get( array $x )
	{
	// fields
		$option = [ 'date' => '__Date__', 'activity' => '__Activity Name__', 'qty' => '__Quantity__' ];
		$values = array_keys( $option );
		for( $ii = 1; $ii <= 3; $ii++ ){
			$x[ 'input-field' . $ii ] = Lib_Html::inputSelect( 'field' . $ii, $values[ $ii-1 ], $option );
		}

	// separator
		$option = [ ',' => ',', ';' => ';', 's' => '__Space__' ];
		$values = array_keys( $option );
		$x['input-separator'] = Lib_Html::inputSelect( 'separator', current($values), $option );
		$x['input-separator']['validate'][] = Lib_Html::validatorRequired();

	// keep current
		$x['input-keep_current'] = Lib_Html::inputCheckbox( 'keep_current', 1 );

	// record
		$x['input-record'] = Lib_Html::inputTextarea( 'record', '' );

		return $x;
	}

	public static function render( array $x )
	{
?>

<form method="post">

<div>
	<div>__Fields__</div>
	<div>
		<div class="pw-grid">
			<?php for( $ii = 1; $ii <= 3; $ii++ ) : ?>
				<div class="pw-col-1-3">
					<label><span>__Field__ #<?php echo esc_html($ii); ?></span><br/><?php Lib_Html::render( $x['input-field' . $ii] ); ?></label>
				</div>
			<?php endfor; ?>
		</div>
	</div>
</div>

<div>
	<div>
		<label for="<?php echo esc_attr( Lib_Html::id($x['input-separator']) ); ?>">__Separate By__</label>
	</div>
	<div><?php Lib_Html::render( $x['input-separator'] ); ?></div>
</div>

<div>
	<div>
		<label for="<?php echo esc_attr( Lib_Html::id($x['input-record']) ); ?>">__Paste Here__</label>
	</div>
	<div><?php Lib_Html::render( $x['input-record'] ); ?></div>
</div>

<div>
	<label>
		<?php Lib_Html::render( $x['input-keep_current'] ); ?> __Keep Current Records__
	</label>
</div>

<div>
	<div>
		<button type="submit" class="button button-primary">__Import Records__</button>
	</div>
</div>

</form>

<?php
	}
}