<?php
/*
Plugin Name: PlainTracker
Plugin URI: https://www.plainware.com/tracker/
Description: A simple tool to track hours spent on your personal and professional projects.
Version: 1.1.0
Author: plainware.com
Author URI: https://www.plainware.com/
Text Domain: plaintracker
Domain Path: /languages/
*/

if( function_exists('add_action') ){
	add_action( 'plugins_loaded', array('Pw3PlainTracker1', 'start') );
}
register_uninstall_hook( __FILE__, array('Pw3PlainTracker1', 'uninstall') );

if( ! class_exists('Pw3PlainTracker1') ){
class Pw3PlainTracker1
{
	public static $instance;

	public $slug = 'plaintracker';
	public $currentPage;
	public $currentPageParams = [];

	public $pages = [];
	public $context = [];

	public static function start()
	{
		new static( __FILE__ );
	}

	public function __construct()
	{
		self::$instance = $this;

		spl_autoload_register( [__CLASS__, 'autoload'] );

		$dev = __DIR__ . '/dev.php';
		if( file_exists($dev) ) require( $dev );

		add_action( 'admin_menu', [$this, 'adminMenu'] );
		add_action( 'admin_init', [$this, 'adminInit'] );
		add_action( 'admin_init', [$this, 'adminHandle'] );
	}

	public function adminMenu()
	{
		$label = 'PlainTracker';
		$cap = 'read';
		$icon = 'dashicons-calendar';

		add_menu_page( $label, $label, $cap, $this->slug, [$this, 'adminRender'], $icon, 31 );

		if( ! isset($_REQUEST['page']) ) return;
		if( $this->slug != sanitize_text_field($_REQUEST['page']) ) return;

	// determine current page
		$cp = '';
		$pageParams = [];

		if( isset($_REQUEST[$this->slug]) ){
			$cp = sanitize_text_field( $_REQUEST[$this->slug] );

		// replace to {id} if seems like id
			$sep = '-';
			$cpa = explode( $sep, $cp );
			for( $ii = 0; $ii < count($cpa); $ii++ ){
				if( ! is_numeric($cpa[$ii]) ) continue;
				$pageParams[] = $cpa[$ii];
				$cpa[$ii] = '{id}';
			}
			$cp = join( $sep, $cpa );
		}
		if( ! $cp ) $cp = 'activity';

		$this->currentPage = $cp;
		$this->currentPageParams = $pageParams;

		do_action( __METHOD__ );
	}

	public function adminInit()
	{
		if( ! $this->currentPage ) return;
		do_action( __METHOD__ );
	}

	public function adminHandle()
	{
		if( ! $this->currentPage ) return;

		$this->pages = \Plaintracker1\Page_Index::pages();
		$this->pages = apply_filters( 'plaintracker_admin_pages', $this->pages );
		if( ! isset($this->pages[$this->currentPage]) ) $this->currentPage = NULL;

		if( ! $this->currentPage ) return;

	// migration up
		\Plaintracker1\Model_Install::up();

		$requestMethod = isset( $_SERVER['REQUEST_METHOD'] ) ? sanitize_text_field($_SERVER['REQUEST_METHOD']) : 'get';
		$requestMethod = strtoupper( $requestMethod );

	// page handler
		$page = $this->pages[ $this->currentPage ];

	// context
		$x = [];
		$x[ '$currentUserId' ] = get_current_user_id();
		$x += $this->currentPageParams;

	// do GET
		if( method_exists($page, 'get') ) $x = $page::get( $x );

	// redirect?
		if( isset($x['redirect']) ){
			do_action( __METHOD__ );
			$this->redirect( $x['redirect'] );
			return;
		}

	// post?
		if( 'POST' === $requestMethod ){
			$post = [];
			foreach( array_keys($_POST) as $k ) $post[$k] = sanitize_text_field( $_POST[$k] );
			$x = $this->grabForm( $x, $post );
			$formValues = $this->formValues( $x );

	// do post if form is valid
			if( FALSE !== $formValues ){
				$x = $page::post( $x, $formValues );
			}
		}

	// redirect?
		if( isset($x['redirect']) ){
			do_action( __METHOD__ );
			$this->redirect( $x['redirect'] );
			return;
		}

	// assets
		$css = [ 'asset/core.css', 'asset/admin.css' ];
		$handleId = 1;
		$appVer = 110;
		foreach( $css as $src ){
			$handle = $this->slug . '-' . $handleId++;
			$src = plugins_url( $src, __FILE__ );
			wp_enqueue_style( $handle, $src, [], $appVer );
		}

	// save context for render
		$this->context = $x;
		do_action( __METHOD__ );
	}

	public function grabForm( array $x, array $post )
	{
		$values = [];
		$prefix = 'input-';

	// grab
		reset( $x );
		foreach( $x as $id => $m ){
			if( $prefix !== substr($id, 0, strlen($prefix)) ) continue;
			$name = substr( $id, strlen($prefix) );
			$value = call_user_func( $m['@grab'], $m, $post );
			$x[$id]['value'] = $value;
			$values[$name] = $value;
		}

	// validate
		reset( $x );
		foreach( $x as $id => $m ){
			if( $prefix !== substr($id, 0, strlen($prefix)) ) continue;
			$name = substr( $id, strlen($prefix) );
			$validators = isset( $m['validate'] ) ? $m['validate'] : [];
			foreach( $validators as $vl ){
				$ok = call_user_func( $vl['@validate'], $vl, $m['value'], $values );
				if( ! $ok ){
					$x[$id]['error'] = $vl['message'];
					break;
				}
			}
		}

		return $x;
	}

	public function formValues( array $x )
	{
		$ret = [];
		$prefix = 'input-';

		reset( $x );
		foreach( $x as $id => $m ){
			if( $prefix !== substr($id, 0, strlen($prefix)) ) continue;
			$name = substr( $id, strlen($prefix) );

			if( strlen($m['error']) ){
				$ret = FALSE;
				break;
			}

			$ret[ $name ] = $m['value'];
		}

		return $ret;
	}

	public function adminRender()
	{
		$currentPage = $this->currentPage;

		if( ! isset($this->pages[$currentPage]) ) return;
		$page = $this->pages[$currentPage];

		$title = method_exists( $page, 'title' ) ? $page::title( $this->context ) : NULL;
		$menu = method_exists( $page, 'menu' ) ? $page::menu( $this->context ) : [];

		$index = $this->pages['index'];
		$topMenu = $index::menu( $this->context );

		$activeTopMenu = NULL;
		foreach( array_keys($topMenu) as $k ){
			if( $k == substr($currentPage, 0, strlen($k) ) ){
				$activeTopMenu = $k;
				break;
			}
		}

		ob_start();
?>

<div class="wrap">
<div id="pw2">

<header>

<?php if( $topMenu ) : ?>
	<div class="nav-tab-wrapper">
		<?php foreach( $topMenu as $k => $label ) : ?>
			<a class="nav-tab<?php if( $activeTopMenu == $k ) : ?> nav-tab-active<?php endif; ?>" href="<?php echo esc_url( add_query_arg([$this->slug => $k], admin_url('admin.php?page=' . $this->slug)) ); ?>"><?php echo esc_html($label); ?></a>
		<?php endforeach; ?>
	</div>
<?php endif; ?>

<?php if( strlen($title) ) : ?>
<h1><?php echo esc_html( $title ); ?></h1>
<?php endif; ?>

<?php if( $menu ) : ?>
	<div class="nav-tab-wrapper">
		<?php foreach( $menu as $k => $label ) : ?>
			<a class="nav-tab" href="<?php echo esc_url( add_query_arg([$this->slug => $k], admin_url('admin.php?page=' . $this->slug)) ); ?>"><?php echo esc_html($label); ?></a>
		<?php endforeach; ?>
	</div>
<?php endif; ?>

</header>

<?php if( method_exists($page, 'render') ) $page::render( $this->context ); ?>

</div>
</div>

<?php
		$ret = ob_get_clean();

	// translate - replace __String__ to __( 'String', 'plaintracker' )
		preg_match_all( '/__[^\=]+__/U', $ret, $ma );
		$count = count($ma[0]);
		for( $ii = 0; $ii < $count; $ii++ ){
			$from = $ma[0][$ii];
			$to = substr( $from, 2, -2 );
			$to = __( $to, 'plaintracker' );
			$ret = str_replace( $from, $to, $ret );
		}

		$ret = apply_filters( __METHOD__, $ret, $this->context );

		echo $ret;
	}

	public function redirect( $to )
	{
		$to = add_query_arg( [$this->slug => $to], esc_url(admin_url('admin.php?page=' . $this->slug)) );
		do_action( __METHOD__, $to );
		wp_redirect( $to );
	}

	public static function uninstall()
	{
		spl_autoload_register( [__CLASS__, 'autoload'] );
		\Plaintracker1\Model_Install::down();
	}

	public static function autoload( $class )
	{
		$namespace = 'Plaintracker1';

		if( $namespace . '\\' !== substr( $class, 0, strlen($namespace) + 1 ) ) return; 

		$f = $class;
		$f = substr( $class, strlen($namespace) + 1 );
		$f = str_replace( '\\', DIRECTORY_SEPARATOR, $f );
		$f = str_replace( '_', DIRECTORY_SEPARATOR, $f );
		$f = __DIR__ . DIRECTORY_SEPARATOR . $f . '.php';

		if( file_exists($f) ){
			require( $f );
		}
	}
}
}